<?php

namespace Lc\ShopBundle\Controller\Admin;

use App\Entity\Product;
use Doctrine\ORM\EntityRepository;
use EasyCorp\Bundle\EasyAdminBundle\Event\EasyAdminEvents;
use Lc\ShopBundle\Context\ProductCategoryInterface;
use Lc\ShopBundle\Context\ProductFamilyInterface;
use Lc\ShopBundle\Context\ProductInterface;
use Lc\ShopBundle\Context\TaxRateInterface;
use Lc\ShopBundle\Form\ProductFamilyCategoriesType;
use Lc\ShopBundle\Form\ProductType;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\CollectionType;
use Symfony\Component\Form\Extension\Core\Type\MoneyType;
use Symfony\Component\HttpFoundation\Response;

class ProductFamilyController extends AdminController
{

        private $taxRateClass;
        private $choicesTaxRateParam;

        public function createEntityFormBuilder($entity, $view)
        {
                $formBuilder = parent::createEntityFormBuilder($entity, $view);

                $class = $this->em->getClassMetadata(ProductCategoryInterface::class);
                $this->taxRateClass = $this->em->getClassMetadata(TaxRateInterface::class);

                //$formBuilder = $this->get('form.factory')->createBuilder(ProductType::class, $entity);*/
                $formBuilder->add('price', MoneyType::class, array(
                        'block_prefix' => 'lc_tax_price',
                        'attr' => array(
                                'class' => 'lc-price'
                        )
                ));
                $formBuilder->add('priceWithTax', MoneyType::class, array(
                        'attr' => array(
                                'class' => 'lc-price-with-tax'
                        ),
                        'required' => false,
                        'mapped' => false
                ));


                //CHOICE qui permet de sélectionner une taxe
                //ça c'est du lourd fais très attention faudra  refactorer
                $this->getUser()->getMerchant()->getTaxRate();
                $choicesTaxRate['TVA valeur par défaut'] = 0;
                foreach ($this->em->getRepository($this->taxRateClass->name)->findAll() as $tax) {
                        $choicesTaxRate[$tax->getTitle()] = $tax->getId();
                        $this->choicesTaxRateParam[$tax->getId()] = $tax->getValue();
                }
                //là mon ami je kiffe symfo !!!!!
                $this->choicesTaxRateParam[0] = $this->getUser()->getMerchant()->getTaxRate()->getValue();


                $formBuilder->add('taxRate', ChoiceType::class, array(
                        'choices' => $choicesTaxRate,
                        'mapped' => false,
                        'choice_attr' => function ($choice, $key, $value) {
                                return ['data-tax-rate-value' => $this->choicesTaxRateParam[$choice]];
                        },
                ));

                $formBuilder->add('unit', ChoiceType::class, array(
                        'choices' => array(
                                'pièce' => 'piece',
                                'g' => 'g',
                                'kg' => 'kg',
                                'ml' => 'mL',
                                'l' => 'L'

                        ),
                ));
                $formBuilder->add('behaviorCountStock', ChoiceType::class, array(
                        'choices' => array(
                                'Gèrer le stock par déclinaison' => 'by-product',
                                'Gèrer le stock par produit' => 'by-product-family'
                        ),
                        'multiple' => false,
                        'expanded' => true
                ));

                $formBuilder->add('productCategories', ProductFamilyCategoriesType::class) ;

                $formBuilder->add('products', CollectionType::class, array(
                                'label' => 'Catégories',
                                'entry_type' => ProductType::class,
                                'allow_add' => true,
                                'prototype' => true
                        )
                );

                return $formBuilder;
        }

        public function updateEntity($entity)
        {
                $prodductFamilyRequest = $this->request->request->get('productfamily');

                if ($taxRateId = intval($prodductFamilyRequest['taxRate']) > 0) {
                        $repo = $this->em->getRepository(TaxRateInterface::class);
                        $entity->setTaxRate($repo->find($taxRateId));
                }

                $this->processCategories($entity) ;
                $this->processProducts($entity) ;

                $this->setUpdated($entity);

                parent::updateEntity($entity);
        }

        public function persistEntity($entity)
        {
                $this->processCategories($entity) ;
                $this->processProducts($entity) ;

                parent::persistEntity($entity) ;
        }

        protected function processProducts($entity)
        {
                if(count($entity->getProducts()) == 0) {
                        $product = new Product();
                        $product->setTitle($entity->getTitle()) ;
                        $product->setCreatedBy($this->getUser()) ;
                        $product->setUpdatedBy($this->getUser()) ;
                        $this->em->persist($product);

                        $entity->addProduct($product) ;
                }
        }

        protected function processCategories(ProductFamilyInterface $entity)
        {
                $productCategoryRepository = $this->getDoctrine()->getRepository(ProductCategoryInterface::class) ;
                $productCategories = $entity->getProductCategories() ;

                $entity->initProductCategories() ;

                foreach($productCategories as $key => $bool) {
                        if(is_bool($bool) && $bool) {
                                if(strpos($key, 'category_children_') !== false) {
                                        $idCategory = (int) str_replace('category_children_', '', $key) ;
                                }
                                else {
                                        $idCategory = (int) str_replace('category_', '', $key) ;
                                }

                                $category = $productCategoryRepository->find($idCategory) ;
                                $entity->addProductCategory($category) ;
                        }
                }
        }
        protected function editAction()
        {

                $this->dispatch(EasyAdminEvents::PRE_EDIT);

                $id = $this->request->query->get('id');
                $easyadmin = $this->request->attributes->get('easyadmin');
                $entity = $easyadmin['item'];

                if ($this->request->isXmlHttpRequest() && $property = $this->request->query->get('property')) {
                        $newValue = 'true' === mb_strtolower($this->request->query->get('newValue'));
                        $fieldsMetadata = $this->entity['list']['fields'];

                        if (!isset($fieldsMetadata[$property]) || 'toggle' !== $fieldsMetadata[$property]['dataType']) {
                                throw new \RuntimeException(sprintf('The type of the "%s" property is not "toggle".', $property));
                        }

                        $this->updateEntityProperty($entity, $property, $newValue);

                        // cast to integer instead of string to avoid sending empty responses for 'false'
                        return new Response((int) $newValue);
                }

                $fields = $this->entity['edit']['fields'];

                $editForm = $this->executeDynamicMethod('create<EntityName>EditForm', [$entity, $fields]);
                $deleteForm = $this->createDeleteForm($this->entity['name'], $id);



                $editForm->handleRequest($this->request);
                if ($editForm->isSubmitted() && $editForm->isValid()) {
                        $this->processUploadedFiles($editForm);

                        $this->dispatch(EasyAdminEvents::PRE_UPDATE, ['entity' => $entity]);
                        $this->executeDynamicMethod('update<EntityName>Entity', [$entity, $editForm]);
                        $this->dispatch(EasyAdminEvents::POST_UPDATE, ['entity' => $entity]);

                        return $this->redirectToReferrer();
                }

                $this->dispatch(EasyAdminEvents::POST_EDIT);

                $productCategoryRepository = $this->getDoctrine()->getRepository(ProductCategoryInterface::class);
                $categories = $productCategoryRepository->findAllParents();

                $parameters = [
                        'form' => $editForm->createView(),
                        'entity_fields' => $fields,
                        'entity' => $entity,
                        'delete_form' => $deleteForm->createView(),
                        'categories' => $categories
                ];

                return $this->executeDynamicMethod('render<EntityName>Template', ['edit', $this->entity['templates']['edit'], $parameters]);
        }

        protected function newAction()
        {
                $this->dispatch(EasyAdminEvents::PRE_NEW);

                $entity = $this->executeDynamicMethod('createNew<EntityName>Entity');

                $easyadmin = $this->request->attributes->get('easyadmin');
                $easyadmin['item'] = $entity;
                $this->request->attributes->set('easyadmin', $easyadmin);

                $fields = $this->entity['new']['fields'];

                $newForm = $this->executeDynamicMethod('create<EntityName>NewForm', [$entity, $fields]);

                $newForm->handleRequest($this->request);

                if ($newForm->isSubmitted() && $newForm->isValid()) {
                        $this->processUploadedFiles($newForm);

                        $this->dispatch(EasyAdminEvents::PRE_PERSIST, ['entity' => $entity]);
                        $this->executeDynamicMethod('persist<EntityName>Entity', [$entity, $newForm]);
                        $this->dispatch(EasyAdminEvents::POST_PERSIST, ['entity' => $entity]);

                        return $this->redirectToReferrer();
                }

                $this->dispatch(EasyAdminEvents::POST_NEW, [
                        'entity_fields' => $fields,
                        'form' => $newForm,
                        'entity' => $entity,
                ]);


                $productCategoryRepository = $this->getDoctrine()->getRepository(ProductCategoryInterface::class);
                $categories = $productCategoryRepository->findAllParents();

                $parameters = [
                        'form' => $newForm->createView(),
                        'entity_fields' => $fields,
                        'entity' => $entity,
                        'categories' => $categories,
                ];

                return $this->executeDynamicMethod('render<EntityName>Template', ['new', $this->entity['templates']['new'], $parameters]);
        }

}